{
  (C) 2003 Dirk Frischalowski, dfrischalowski@del-net.com, all rights reserved.

  Version 1.0 25.02.2003
}

unit DFAniCursor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TDFAniCursorString = type string;

  TDFAniCursor = class(TComponent)
  private
    { Private-Deklarationen }
    FDataSize: Integer;
    FFileName: TDFAniCursorString; // new Typ used for PropertyEditor
    FData: Pointer;
    procedure SetFileName(const vFileName: TDFAniCursorString);
    procedure ReadData(Stream: TStream);
    procedure WriteData(Stream: TStream);
  protected
    { Protected-Deklarationen }
    function GetRandomTempFileName : String;
    function GetTempDir : String;
    procedure DefineProperties(Filer: TFiler); override;
  public
    { Public-Deklarationen }
    function SetAniCursor(CursorID: Integer): boolean;
    function IsEmpty: Boolean;
    destructor Destroy; override;
    procedure LoadFromFile(vFileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    procedure SaveToFile(vFileName: string);
  published
    { Published-Deklarationen }
    property AniCursorFile: TDFAniCursorString read FFileName write SetFileName;
  end;

implementation

function TDFAniCursor.SetAniCursor(CursorID: Integer): boolean;
var
  hRes : THandle;
  tmpfile : string;
begin
  Result := false;

  try
    tmpFile := GetRandomTempFileName;
    SaveToFile(tmpFile);
    hRes := LoadImage (0, PChar(tmpFile), IMAGE_CURSOR, 0, 0, LR_DEFAULTSIZE or LR_LOADFROMFILE);
    DeleteFile(tmpfile);
    Screen.Cursors[CursorID] := hRes;
  except
    exit;
  end;

  Result := true;
end;

function TDFAniCursor.GetRandomTempFileName : String;
var
 S : String;
begin
  SetLength(S, MAX_PATH);
  GetTempFileName(PChar(GetTempDir), 'tmp', 0, PChar(S));
  Result := StrPas(PChar(S));
end;

function TDFAniCursor.GetTempDir : String;
var
 cb : DWORD;
 S : String;
begin
  SetLength(S, MAX_PATH);
  cb := MAX_PATH;
  If GetTempPath(cb, PChar(S)) = 0 Then
    begin
      SetLength (S, 1024);
      cb := 1023;
      GetWindowsDirectory (PChar(S), cb);
    end;
  S := StrPas (PChar(S));
  If S[Length (S)] <> '\' Then
     S := S + '\';
  Result := S;
end;

function TDFAniCursor.IsEmpty: Boolean;
begin
  Result := FDataSize = 0;
end;

destructor TDFAniCursor.Destroy;
begin
  if not IsEmpty then
    FreeMem(FData, FDataSize);
  inherited Destroy;
end;

procedure TDFAniCursor.SetFileName(const vFileName: TDFAniCursorString);
begin
  if vFileName <> '' then
  begin
    FFileName := ExtractFileName(vFileName);
    if (not(csLoading in ComponentState)) and FileExists(vFileName) then
      LoadFromFile(vFileName);
  end
  else
  begin
    FFileName := '';
    if not IsEmpty then
      FreeMem(FData, FDataSize);
    FDataSize := 0;
  end;
end;

procedure TDFAniCursor.DefineProperties(Filer: TFiler);
  function DoWrite: Boolean;
  begin
    if Filer.Ancestor <> nil then
      Result := not (Filer.Ancestor is TDFAniCursor)
    else
      Result := not IsEmpty;
  end;
begin
  inherited DefineProperties(Filer);

  Filer.DefineBinaryProperty('Data', ReadData, WriteData, DoWrite);
end;

procedure TDFAniCursor.LoadFromFile(vFileName: string);
var
  F: TFileStream;
begin
  F := TFileStream.Create(vFileName, fmOpenRead);
  try
    LoadFromStream(F);
  finally
    F.Free;
  end;
end;

procedure TDFAniCursor.LoadFromStream(Stream: TStream);
begin
  if not IsEmpty then
    FreeMem(FData, FDataSize);
  FDataSize := 0;
  FData := AllocMem(Stream.Size);
  FDataSize := Stream.Size;
  Stream.Read(FData^, FDataSize);
end;

procedure TDFAniCursor.ReadData(Stream: TStream);
begin
  LoadFromStream(Stream);
end;

procedure TDFAniCursor.WriteData(Stream: TStream);
begin
  SaveToStream(Stream);
end;

procedure TDFAniCursor.SaveToStream(Stream: TStream);
begin
  if not IsEmpty then
    Stream.Write(FData^, FDataSize);
end;

procedure TDFAniCursor.SaveToFile(vFileName: string);
var
  F: TFileStream;
begin
  F := TFileStream.Create(vFileName, fmCreate);
  try
    SaveToStream(F);
  finally
    F.Free;
  end;
end;

end.
